unit GameObjectUnit;

interface

uses
  SysUtils,
  // JEDI-SDL
  sdl,
  sdlutils,

  GraphicsUnit,
  GameConstantsUnit;

type
  TTank = class(TObject)
    X, Y: Real;
    TrackSize: Cardinal;
    TurretX, TurretY,
    TurretLength: Integer;
    AimAngle, AimPower: Real;
    Color: Cardinal;
    Sprite: PSDL_Surface;
    constructor Init(oTrackSize: Cardinal; oTurretX, oTurretY, oTurretLength: Integer; oColor: Cardinal; ImageFile: String);
    procedure Draw(GameScreen: PSDL_Surface);
  end;
  TBattlefield = class(TObject)
    Width, Height: Integer;
    LandHeight: Array[0 .. 1024] of Integer;
    LandColor, SkyColor: Cardinal;

    isBGImage: Boolean;
    Background: PSDL_Surface; // Background Graphic

    constructor Init(ScreenWidth, ScreenHeight: Integer; Land, Sky: Cardinal; useBGImage: Boolean; BGImageFile: String);
    procedure GenerateLand(Highest, Lowest, Variation: Integer);
    procedure SmoothenLand(SmoothSize: Integer); // Tries to smoothen rough generated land!
    procedure PlaceTank(x, gap: Integer; var Tank: TTank);
    procedure ScatterTanks(var Tanks: Array of TTank; NumberOfTanks, MinTankDist, DirtGap: Integer);
    procedure DrawSky(GameScreen: PSDL_Surface);
    procedure DrawLand(GameScreen: PSDL_Surface);
  end;


implementation


// TTank
constructor TTank.Init(oTrackSize: Cardinal; oTurretX, oTurretY, oTurretLength: Integer; oColor: Cardinal; ImageFile: String);
begin
    X := 0;
    Y := 0;
    TrackSize := oTrackSize;

    TurretX := oTurretX;
    TurretY := oTurretY;
    TurretLength := oTurretLength;

    AimAngle := 45;  // Default Value
    AimPower := 500; // Default Value

    Color := oColor;

    // Load Sprite File
    Sprite := LoadImage('images/' + ImageFile, True);
end;
procedure TTank.Draw(GameScreen: PSDL_Surface);
var
  SrcRect: TSDL_Rect;
  DestRect: TSDL_Rect;
begin
     // Draw Turret
     SDL_DrawLine(GameScreen, Round(X + TurretX), Round(GameScreen.h - 1 - Y + TurretY),
                  Round(X + TurretX + TurretLength), Round(GameScreen.h - 1 - Y + TurretY), Color);
     // Draw Body
     SrcRect := SDLRect(0, 0, Sprite.w, Sprite.h);
     DestRect := SDLRect(Round(X - Sprite.w / 2), Round(GameScreen.h - 1 - Y - Sprite.h), Sprite.w, Sprite.h);
     SDL_BlitSurface(Sprite, @SrcRect, GameScreen, @DestRect);
end;
// End of TTank


// TBattlefield
constructor TBattlefield.Init(ScreenWidth, ScreenHeight: Integer; Land, Sky: Cardinal; useBGImage: Boolean; BGImageFile: String);
begin
     Width := ScreenWidth;
     Height := ScreenHeight;

     LandColor  := Land;
     SkyColor   := Sky;
     isBGImage := useBGImage;

     // Load Background
     if (useBGImage) then
        Background := LoadImage('images/' + BGImageFile, False);
end;
procedure TBattlefield.GenerateLand(Highest, Lowest, Variation: Integer);
var i: Integer;
    rand: Real;
begin
     LandHeight[0] := Lowest + Round(Random * (Highest - Lowest));
     for i := 1 to Width - 1 do
     begin
          repeat
            rand := Random;
            LandHeight[i] := Round(LandHeight[i - 1] + (rand * Variation) - Variation / 2);
            if (LandHeight[i] < Lowest) then
               LandHeight[i] := Lowest;
          until (LandHeight[i] <= Highest);
     end;
end;
procedure TBattlefield.SmoothenLand(SmoothSize: Integer);
var i, j: Integer;
    Mass, NumOfMassSamples: Integer;
begin
     for i := 0 to Width - 1 do
     begin
          // Get average height of selected area...
          Mass := 0;
          NumOfMassSamples := 0;
          for j := i - SmoothSize to i + SmoothSize do
              if (j > 0) and (j < Width - 1) then // Samples must be in bounds!
              begin
                   inc(NumOfMassSamples);
                   Mass := Mass + LandHeight[j];
              end;

          // Resize LandHeight element
          LandHeight[i] := Round(Mass / NumOfMassSamples);
     end;
end;
procedure TBattlefield.PlaceTank(x, gap: Integer; var Tank: TTank);
var
  i: Integer;
  Lowest: Cardinal;
begin
     // Snap Tank back on-screen
     if (x - Tank.TrackSize < 0) then
        x := Tank.TrackSize;
     if (x + Tank.TrackSize > Width - 1) then
        x := Width - 1 - Tank.TrackSize;

     // Find lowest part of the land
     Lowest := LandHeight[x];
     for i := x - Tank.TrackSize to x + Tank.TrackSize do
         if (LandHeight[i] < Lowest) then
            Lowest := LandHeight[i];

     // Level the ground around the tank
     for i := x - Tank.TrackSize - gap to x + Tank.TrackSize + gap do
         if (i >= 0) or (i <= Width - 1) then
            LandHeight[i] := Lowest;

     // Relocate Tank
     Tank.X := x;
     Tank.Y := Lowest;
end;
procedure TBattlefield.ScatterTanks(var Tanks: Array of TTank; NumberOfTanks, MinTankDist, DirtGap: Integer);
var
  i, j: Integer;
  x: Integer;
  SpaceClear: Boolean;
begin
     for i := 0 to NumberOfTanks - 1 do
     begin
          repeat
            SpaceClear := True;

            // Generate random location
            x := Random(Width);
            
            // Check if location is off screen
            if (x - Tanks[i].TrackSize < 0) or
               (x + Tanks[i].TrackSize > Width - 1) then
               SpaceClear := False;

            // Check with other tanks
            for j := 0 to NumberOfTanks - 1 do
                if (i <> j) then
                begin
                     // Location taken by other tank
                     if ((x + Tanks[i].TrackSize >= Tanks[j].X - Tanks[j].TrackSize - MinTankDist) and
                         (x + Tanks[i].TrackSize <= Tanks[j].X + Tanks[j].TrackSize + MinTankDist)) or
                        ((x - Tanks[i].TrackSize >= Tanks[j].X - Tanks[j].TrackSize - MinTankDist) and
                         (x - Tanks[i].TrackSize <= Tanks[j].X + Tanks[j].TrackSize + MinTankDist)) then
                        SpaceClear := False;
                end;
          until (SpaceClear);
          // Place Tank
          PlaceTank(x, DirtGap, Tanks[i]);
     end;
end;
procedure TBattlefield.DrawLand(GameScreen: PSDL_Surface);
var
  i: Integer;
begin
     // Land
     for i := 0 to Width - 1 do
         SDL_DrawLine(GameScreen, i, Height - 1, i, Height - 1 - LandHeight[i], LandColor);
end;
procedure TBattlefield.DrawSky(GameScreen: PSDL_Surface);
begin
     if (isBGImage) then
         DrawBackgound(GameScreen, Background)
     else
         SDL_FillRect(GameScreen, PSDLRect(0, 0, 800, 600), SkyColor);
end;
// End of TBattlefield

begin
end.
